# -*- python -*-
# $RCSfile: pinnodesPage.py,v $
# $Revision: 1.17 $
# $Author: langer $
# $Date: 2006-12-19 20:29:44 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import utils
from oof2.common.IO import mainmenu
from oof2.common.IO import whoville
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import skeletoncontext
from oof2.engine.IO import pinnodes
from oof2.engine import pinnodesmodifier
import gtk
import types, string

# Define some convenience variables.
OOF = mainmenu.OOF

## TODO: Desensitize OK button when Skeleton is busy

class PinNodesPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Pin Nodes", ordering=120.1,
                                 tip='Pin and unpin nodes')

        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.skelwidget = whowidget.WhoWidget(whoville.getClass('Skeleton'),
                                              callback=self.select_skeletonCB)
        label = gtk.Label('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.skelwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label('Skeleton=')
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.skelwidget.gtk[1], expand=0, fill=0)

        mainpane = gtk.HPaned()
        gtklogger.setWidgetName(mainpane, 'Pane')
        mainbox.pack_start(mainpane, expand=1, fill=1)

        # Pinned nodes status in the left half of the main pane
        pnsframe = gtk.Frame("Pinned Nodes Status")
        gtklogger.connect_passive(pnsframe, 'size-allocate')
        pnsframe.set_shadow_type(gtk.SHADOW_IN)
        self.datascroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(self.datascroll, "StatusScroll")
        pnsframe.add(self.datascroll)
        self.datascroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.psdata = gtk.TextView()
        self.psdata.set_editable(False)
        self.psdata.set_wrap_mode(gtk.WRAP_WORD)
        self.psdata.set_cursor_visible(False)
        self.datascroll.add_with_viewport(self.psdata)
        mainpane.pack1(pnsframe, resize=1, shrink=0)
        
        # Pin nodes method
        modframe = gtk.Frame("Pin Nodes Methods")
        gtklogger.setWidgetName(modframe, 'Modify')
        modframe.set_shadow_type(gtk.SHADOW_IN)
        modbox = gtk.VBox()  # will have "methods" and "buttons"
        modframe.add(modbox)
##        scroll = gtk.ScrolledWindow()
##        modbox.pack_start(scroll, expand=1, fill=1)
##        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.pinModFactory = regclassfactory.RegisteredClassFactory(
            pinnodesmodifier.PinNodesModifier.registry,
            title="Method:", scope=self, name="Method")
##        scroll.add_with_viewport(self.pinModFactory.gtk)
        modbox.pack_start(self.pinModFactory.gtk, expand=1, fill=1, padding=2)

        # buttons
        hbox1 = gtk.HBox()
        modbox.pack_start(hbox1, expand=0, fill=0, padding=2)
        self.okbutton = gtk.Button(stock=gtk.STOCK_OK)
        gtklogger.setWidgetName(self.okbutton, 'OK')
        gtklogger.connect(self.okbutton, "clicked", self.okCB)
        guitop.top().tooltip(self.okbutton, "Pin nodes with the selected method.")
        self.undobutton = gtk.Button(stock=gtk.STOCK_UNDO)
        gtklogger.setWidgetName(self.undobutton, 'Undo')
        gtklogger.connect(self.undobutton, "clicked", self.undoCB)
        guitop.top().tooltip(self.undobutton, "Undo the latest action.")
        self.redobutton = gtk.Button(stock=gtk.STOCK_REDO)
        gtklogger.setWidgetName(self.redobutton, 'Redo')
        gtklogger.connect(self.redobutton, "clicked", self.redoCB)
        guitop.top().tooltip(self.redobutton, "Redo the latest undone action.")
        hbox1.pack_start(self.undobutton, expand=0, fill=1, padding=2)
        hbox1.pack_start(self.okbutton, expand=1, fill=1, padding=2)
        hbox1.pack_end(self.redobutton, expand=0, fill=1, padding=2)

        hbox2 = gtk.HBox(homogeneous=1)
        modbox.pack_start(hbox2, expand=0, fill=0, padding=2)
        self.unpinallbutton = gtk.Button("Unpin All")
        gtklogger.setWidgetName(self.unpinallbutton, 'Unpin All')
        gtklogger.connect(self.unpinallbutton, "clicked", self.unpinallCB)
        guitop.top().tooltip(self.unpinallbutton, "Unpin all the pinned nodes.")
        self.invertbutton = gtk.Button("Invert")
        gtklogger.setWidgetName(self.invertbutton, 'Invert')
        gtklogger.connect(self.invertbutton, "clicked", self.invertCB)
        guitop.top().tooltip(self.invertbutton, "Invert - pin the unpinned and unpin the pinned.")
        hbox2.pack_start(self.unpinallbutton, expand=1, fill=1, padding=2)
        hbox2.pack_start(self.invertbutton, expand=1, fill=1, padding=2)
        
        mainpane.pack2(modframe, resize=0, shrink=0)

        # Switchboard callbacks
        switchboard.requestCallbackMain(('who changed', 'Skeleton'),
                                        self.changeSkeleton)
        switchboard.requestCallbackMain(('new who', 'Microstructure'),
                                        self.newMS)
        switchboard.requestCallbackMain("new pinned nodes",
                                        self.newNodesPinned)
        switchboard.requestCallbackMain(self.skelwidget,
                                        self.skel_update)

        # Initial sensitizing
        self.sensitize()

    # Switchboard callbacks
    def changeSkeleton(self, skelcontext):
        try:
            path = skelcontext.path()
        except KeyError:
            return
        if self.skelwidget.get_value()==path:
            self.set_state(path)

    def newMS(self, msname):
        if not self.skelwidget.get_value():
            self.skelwidget.set_value(msname)

    def newNodesPinned(self, pinnednodes, skelcontext):
        try:
            path = skelcontext.path()
        except KeyError:
            return
        if self.skelwidget.get_value()==path:
            self.update(path)

    def skel_update(self, *args, **kwargs):  # Switchboard "self.skelwidget"
        skeleton = self.skelwidget.get_value()
        self.update(skeleton)

    # setting the state of "self.skelwidget"
    def set_state(self, skeleton):
        if labeltree.makePath(skeleton) == \
               labeltree.makePath(self.skelwidget.get_value()):
            self.update(skeleton)
        else:
            self.skelwidget.set_value(skeleton)
            # set_value triggers the "self.skelwidget" switchboard
            # callback, which calls "self.update".
            
    # update pinned nodes status
    def update(self, skeleton):
        debug.mainthreadTest()
        if skeleton is not None:
            skelpath = labeltree.makePath(skeleton)
            if skelpath[-1]:  # path is complete
                skelcontext = skeletoncontext.skeletonContexts[skeleton]
                skel = skelcontext.getObject()
                nnodes = skel.nnodes()
                npinned = skelcontext.pinnednodes.npinned()
                self.psdata.get_buffer().set_text(
                    "Total No. of Nodes: %d\nNo. of Pinned Nodes: %d\n" %
                    (nnodes, npinned)
                    )
        self.sensitize()

    def select_skeletonCB(self, *args):
        self.set_state(self.skelwidget.get_value())

    def currentSkelContext(self):
        try:
            return skeletoncontext.skeletonContexts[self.skelwidget.get_value()]
        except KeyError:
            return None

    def currentSkelName(self):
        return self.skelwidget.get_value()
    
    def sensitize(self):
        debug.mainthreadTest()
        skelcontext = self.currentSkelContext()
        if not not skelcontext:
            n = skelcontext.pinnednodes.npinned()
        self.okbutton.set_sensitive(not not skelcontext)
        self.undobutton.set_sensitive(not not skelcontext and \
                                      skelcontext.pinnednodes.undoable())
        self.redobutton.set_sensitive(not not skelcontext and \
                                      skelcontext.pinnednodes.redoable())
        self.unpinallbutton.set_sensitive(not not skelcontext and n > 0)
        self.invertbutton.set_sensitive(not not skelcontext)


    def okCB(self, *args):
        modmethod = self.pinModFactory.getRegistration()
        if modmethod is not None:
            self.pinModFactory.set_defaults()
            menuitem = getattr(mainmenu.OOF.Skeleton.PinNodes,
                               utils.space2underscore(modmethod.name()))
            menuitem.callWithDefaults(skeleton=self.currentSkelName())

    def undoCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.Undo(skeleton=self.currentSkelName())
    
    def redoCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.Redo(skeleton=self.currentSkelName())

    def unpinallCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.UnpinAll(skeleton=self.currentSkelName())

    def invertCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.Invert(skeleton=self.currentSkelName())


# Create the page.
pinnodesPage = PinNodesPage()
