// -*- C++ -*-
// $RCSfile: compoundsubproblem.h,v $
// $Revision: 1.4 $
// $Author: langer $
// $Date: 2007-01-16 22:42:55 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef COMPOUNDSUBPROBLEM_H
#define COMPOUNDSUBPROBLEM_H

#include "engine/predicatesubproblem.h"
#include <iostream>

class BinarySubproblemPredicate {
protected:
  CSubProblem *subA;
  CSubProblem *subB;
public:
  BinarySubproblemPredicate(CSubProblem*, CSubProblem*);
  virtual ~BinarySubproblemPredicate() {}
  virtual bool operator()(const FEMesh*, const Element*) = 0;
};

class UnionSBPredicate: public BinarySubproblemPredicate {
public:
  UnionSBPredicate(CSubProblem *a, CSubProblem *b)
    : BinarySubproblemPredicate(a, b)
  {}
  virtual bool operator()(const FEMesh*, const Element*);
};

class UnionSubProblem: public PredicateSubProblem<UnionSBPredicate> {
public:
  UnionSubProblem(CSubProblem *a, CSubProblem *b)
    : PredicateSubProblem<UnionSBPredicate>(UnionSBPredicate(a, b))
  {}
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

class IntersectionSBPredicate: public BinarySubproblemPredicate {
public:
  IntersectionSBPredicate(CSubProblem *a, CSubProblem *b)
    : BinarySubproblemPredicate(a, b)
  {}
  virtual bool operator()(const FEMesh*, const Element*);
};

class IntersectionSubProblem:
  public PredicateSubProblem<IntersectionSBPredicate>
{
public:
  IntersectionSubProblem(CSubProblem *a, CSubProblem *b)
    : PredicateSubProblem<IntersectionSBPredicate>(IntersectionSBPredicate(a,b))
  {}
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

class XorSBPredicate: public BinarySubproblemPredicate {
public:
  XorSBPredicate(CSubProblem *a, CSubProblem *b)
    : BinarySubproblemPredicate(a, b)
  {}
  virtual bool operator()(const FEMesh*, const Element*);
};

class XorSubProblem:
  public PredicateSubProblem<XorSBPredicate>
{
public:
  XorSubProblem(CSubProblem *a, CSubProblem *b)
    : PredicateSubProblem<XorSBPredicate>(XorSBPredicate(a,b))
  {}
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

class ComplementSBPredicate {
protected:
  CSubProblem *complement;
public:
  ComplementSBPredicate(CSubProblem*);
  bool operator()(const FEMesh*, const Element*);
};

class ComplementSubProblem: public PredicateSubProblem<ComplementSBPredicate> {
public:
  ComplementSubProblem(CSubProblem *comp)
    : PredicateSubProblem<ComplementSBPredicate>(ComplementSBPredicate(comp))
  {}
};

#endif // COMPOUNDSUBPROBLEM_H
