// -*- C++ -*-
// $RCSfile: timestepadapter.C,v $
// $Revision: 1.12 $
// $Author: langer $
// $Date: 2006-12-19 20:29:44 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modifed
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "engine/csrmatrix.h"
#include "engine/csubproblem.h"
#include "engine/matvec.h"
#include "engine/solver.h"
#include "engine/solverdriver.h"
#include "engine/timestepadapter.h"
#include "engine/steppingscheme.h"



TimeStepAdapter::~TimeStepAdapter()
{
  if (x_prev)
    {
      delete x_prev;
    }
}


void TimeStepAdapter::accept(CSubProblem *subproblem, SteppingScheme *smethod) {
  subproblem->update_time(get_dt());
  if(x_prev == 0) {
    x_prev = new VECTOR_D (smethod->get_x()->size(), 0.0);
  }
  else {
    if(x_prev->size() != smethod->get_x()->size())
      x_prev->newsize(smethod->get_x()->size());
  }
  (*x_prev) = (*smethod->get_x());
  subproblem->set_unknowns(*x_prev, subproblem->freedofmap);
}

void TimeStepAdapter::reject(CSubProblem *subproblem) {
  //time step is NOT updated.
  //in fact, old solution is restored
  if (x_prev)
    subproblem->set_unknowns(*x_prev, subproblem->freedofmap);
}


double NullAdapter::get_residual(SteppingScheme *smethod) {
  return 0; //returns ZERO
}

void NullAdapter::accept(CSubProblem *subproblem, SteppingScheme *smethod) {
  subproblem->update_time(get_dt());
}

void NullAdapter::reject(CSubProblem*) {
}

bool NullAdapter::is_okay(SteppingScheme *smethod) {
  return true;
}

void NullAdapter::set_time_step(CSubProblem *subproblem,
				SteppingScheme *smethod)
{
  // first check current time, t
  double curr_time = subproblem->current_time();
  // next check where it will be after dt
  double curr_dt = smethod->get_dt();
  double future_time = curr_time + curr_dt;
  // if t+dt>tf, set dt so that t+dt==tf
  if (future_time> smethod->get_tf()) {
    // subtract the time it would overshoot.
    smethod->set_dt(curr_dt - (future_time  - smethod->get_tf()));
    continew = false;		// force it to stop at the end
  }
  dt = smethod->get_dt();
  //std::cerr << "NullAdapter::set_time_step t=" << curr_time << " dt = " << dt << std::endl;
}
