// -*- C++ -*-
// $RCSfile: imageops.C,v $
// $Revision: 1.1 $
// $Author: olgak $
// $Date: 2007-07-20 17:50:57 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "imageops.h"
#include <iostream>
#include <math.h>

#include "image/oofimage.h" //This is where threshold is located
//#include "image/GRAINBDY/close.h"
//#include "image/GRAINBDY/skeletonize.h"
#include "image/oofimage.h"
#include "common/array.h"
#include "common/doublearray.h"
#include "common/boolarray.h"
#include "common/intarray.h"
#include "common/random.h"
#include "image/oofimage.h"

void setFromArray(OOFImage& colorImage,const DoubleArray& image) {
//Creates a color image from an array of doubles.
  colorImage.set(image,gray2color);
}


void setFromInt(OOFImage& colorImage,const IntArray& image) {
//Creates a color image from an array of doubles.
  colorImage.set(image,int2color);
}

DoubleArray arrayFromImage(OOFImage& colorImage){
	return colorImage.convert(togray);
}

/*
Places all of the checked pixels on the old image to see if the borders match up.
*/
DoubleArray mountThresholdedImageOnOldImage(DoubleArray gray, DoubleArray newgray){
	ICoord size = gray.size();
	ICoord center;
	for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
		center = i.coord();
		if (newgray[center] == 1)
			gray[center] = 1;
	
	}
	return gray;
}

void drawFixerLines(DoubleArray &gray, int x0, int y0, int x1, int y1, int color){
        int dy = y1 - y0;
        int dx = x1 - x0;
        int stepx, stepy;
	int width = 1;
	ICoord size = gray.size();
        if (dy < 0) { dy = -dy;  stepy = -width; } else { stepy = width; }
        if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
        dy <<= 1;
        dx <<= 1;

        y0 *= width;
        y1 *= width;
	if (pixelInBounds(ICoord(y0, x0), size))
	        gray[ICoord(y0, x0)] = color; 
        if (dx > dy) {
            int fraction = dy - (dx >> 1);
            while (x0 != x1) {
                if (fraction >= 0) {
                    y0 += stepy;
                    fraction -= dx;
                }
                x0 += stepx;
                fraction += dy;
                if (pixelInBounds(ICoord(y0, x0), size))
	      		gray[ICoord(y0, x0)] = color; 
            }
        } else {
            int fraction = dx - (dy >> 1);
            while (y0 != y1) {
                if (fraction >= 0) {
                    x0 += stepx;
                    fraction -= dy;
                }
                y0 += stepy;
                fraction += dx;
                if (pixelInBounds(ICoord(y0, x0), size))
	        	gray[ICoord(y0, x0)] = color;  
            }
        }
	
}

/*
Normalize the intensity of the image to range from 0 to 1. 
*/
DoubleArray normalizeImage(DoubleArray original){
	ICoord size = original.size();
	double min = original[ICoord(0,0)];
	double max = original[ICoord(0,0)];
	for (DoubleArray::iterator i = original.begin();i !=original.end(); ++i){
		ICoord curr = i.coord();
		double a = original[curr];
		if (a > max)
			max = a;
		if (a < min)
			min = a;
	}
	double difference = max - min;
	double toadd = 0;
	if (min < 0){
		toadd = -1*min; // will be posisite amount to add
	}
	else
		toadd = -1*min; // will be negative amount to add so it will subtract

	if (max > 1 || min < 0){
		for (DoubleArray::iterator i = original.begin();i !=original.end(); ++i){
			ICoord curr = i.coord();
			original[curr] = (original[curr] + toadd)/difference;
		}
	}	
	return original;
}

bool pixelInBounds(ICoord pxl, ICoord size) {
  int xx = (pxl)(0);
  int yy = (pxl)(1);
  if ( (xx<0) || (xx>=size(0)) || (yy<0) || (yy>=size(1)) )
    return false;
  return true;
}
