 /* -*- C++ -*-
 $RCSfile: skeletonize.C,v $
 $Revision: 1.3 $
 $Author: langer $
 $Date: 2007-07-20 20:29:09 $

 This software was produced by NIST, an agency of the U.S. government,
 and by statute is not subject to copyright in the United States.
 Recipients of this software assume all responsibilities associated
 with its operation, modification and maintenance. However, to
 facilitate maintenance we ask that before distributing modified
 versions of this software, you first contact the authors at
 oof_manager@ctcms.nist.gov. */ 
 
 
#include "skeletonize.h"
#include "fixborders.h"
 
 
DoubleArray Skeletonize::thinEdges(DoubleArray gray){
  ICoord size = gray.size();;
  DoubleArray a = DoubleArray(gray);
  DoubleArray b = DoubleArray(gray);
  FixBorders f = FixBorders();
  int numDeleted = 0;
  do{
    numDeleted = 0;
    for (DoubleArray::iterator i = a.begin();i !=a.end(); ++i){
      ICoord center = i.coord();
      if (a[center] == 1){
	int numFull = numNonZeroTransitions(center, a);
	int num = f.numSideEmpty(center, a, 1, 1);
	if (2 <= num && num <= 6 && numFull == 1){
				
	  if (pixelInBounds(center + ICoord(0,1),size) &&
	      pixelInBounds(center + ICoord(0,-1),size) &&
	      pixelInBounds(center + ICoord(1,0),size) &&
	      pixelInBounds(center + ICoord(-1,0),size)) {
	    if ((a[center + ICoord(0,1)] == 0) ||
		(a[center + ICoord(1,0)] == 0) ||
		(a[center + ICoord(0,-1)] == 0)) {
	      if ((a[center + ICoord(1,0)] == 0) ||
		  (a[center + ICoord(0,-1)] == 0) ||
		  (a[center + ICoord(-1,0)] == 0)) {
		if (pixelInBounds(center + ICoord(-1,-1),size) &&
		    a[center + ICoord(-1,-1)] != 0) {
		  ++numDeleted;
		  b[center] = 0;
		}
	      }
						
	    }
	  }
	}
				
				
      }
    }
    a = b;
    for (DoubleArray::iterator i = a.begin();i !=a.end(); ++i){
      ICoord center = i.coord();
      if (a[center] == 1){
	int numFull = numNonZeroTransitions(center, a);
	int num = f.numSideEmpty(center, a, 1, 1);
	if (2 <= num && num <= 6 && numFull == 1){
	  if (pixelInBounds(center + ICoord(0,1),size) &&
	      pixelInBounds(center + ICoord(0,-1),size) &&
	      pixelInBounds(center + ICoord(1,0),size) &&
	      pixelInBounds(center + ICoord(-1,0),size)){
	    if ((a[center + ICoord(0,1)] == 0) ||
		(a[center + ICoord(1,0)] == 0) ||
		(a[center + ICoord(-1,0)] == 0)){
	      if ((a[center + ICoord(1,0)] == 0) ||
		  (a[center + ICoord(-1,0)] == 0) ||
		  (a[center + ICoord(0,-1)] == 0)){
		if (pixelInBounds(center + ICoord(1,1),size) &&
		    a[center + ICoord(1,1)] != 0){
		  ++numDeleted;
		  b[center] = 0;
		}
	      }
						
	    }
	  }
	}
      }
    }
    a = b;
  }while(numDeleted != 0);
  return a;	
}

int  Skeletonize::numNonZeroTransitions(ICoord center, DoubleArray gray){
  int num = 0;
  ICoord size = gray.size();
  if (pixelInBounds(center + ICoord(0,1),size) && pixelInBounds(center + ICoord(-1,1),size) && ((gray[center + ICoord(0,1)] == 0 && gray[center + ICoord(-1,1)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(-1,1),size) && pixelInBounds(center + ICoord(-1,0),size) && ((gray[center + ICoord(-1,1)] == 0 && gray[center + ICoord(-1,0)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(-1,0),size) && pixelInBounds(center + ICoord(-1,-1),size) && ((gray[center + ICoord(-1,0)] == 0 && gray[center + ICoord(-1,-1)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(-1,-1),size) && pixelInBounds(center + ICoord(0,-1),size) && ((gray[center + ICoord(-1,-1)] == 0 && gray[center + ICoord(0,-1)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(0,-1),size) && pixelInBounds(center + ICoord(1,-1),size) && ((gray[center + ICoord(0,-1)] == 0 && gray[center + ICoord(1,-1)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(1,-1),size) && pixelInBounds(center + ICoord(1,0),size) && ((gray[center + ICoord(1,-1)] == 0 && gray[center + ICoord(1,0)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(1,0),size) && pixelInBounds(center + ICoord(1,1),size) && ((gray[center + ICoord(1,0)] == 0 && gray[center + ICoord(1,1)] == 1))){ 
    ++num;
  }
  if (pixelInBounds(center + ICoord(1,1),size) && pixelInBounds(center + ICoord(0,1),size) && ((gray[center + ICoord(1,1)] == 0 && gray[center + ICoord(0,1)] == 1))){ 
    ++num;
  }
  return num;
}
 
 
/*
  Returns the number of sides of the center pixel that are of color type
*/
int Skeletonize::numBorderSides(ICoord center, DoubleArray list, int type){
  int num = 0;
  ICoord curr;
  ICoord size = list.size();
  curr = center+ICoord(1,0);
  if (pixelInBounds(curr,size) == true){
    if (list[curr] == type)
      ++num;
  }
  else
    ++num;
  curr = center+ICoord(0,1);
  if (pixelInBounds(curr,size) == true){
    if (list[curr] == type)
      ++num;
  }
  else
    ++num;
  curr = center+ICoord(-1,0);
  if (pixelInBounds(curr,size) == true){
    if (list[curr] == type)
      ++num;
  }
  else
    ++num;
  curr = center+ICoord(0,-1);
  if (pixelInBounds(curr,size) == true){
    if (list[curr] == type)
      ++num;
  }
  else
    ++num;
  return num;

}
 
 
 
bool Skeletonize::pixelInBounds(ICoord pxl, ICoord size) {
  int xx = (pxl)(0);
  int yy = (pxl)(1);
  if ( (xx<0) || (xx>=size(0)) || (yy<0) || (yy>=size(1)) )
    return false;
  return true;
}

 
/*
  After this, not written by me.
*/
 
 
bool Skeletonize::isContour(const BoolArray &image,const ICoord pt) {
  if(image[pt]==1 && (pt(0)==0 || pt(0)==image.width()-1 || pt(1)==0 || pt(1)==image.height()-1))
    return true;
  if (image[pt]==1) {  // If the point is white
    for (int a = -1; a <= 1; a++) {  // Twice
      for (int b = -1; b <= 1; b++) {  // Twice
	if (image[pt+ICoord(a,b)] == 0)
	  return true;  // Returns true if any of the points surrounding the white one is black
      }
    }
  }
  return false;
}

bool Skeletonize::isDeletable(const BoolArray& image,ICoord pt) {
  int numTrans=0;
  int xChange[9]={-1,0,1,1,1,0,-1,-1,-1};
  int yChange[9]={1,1,1,0,-1,-1,-1,0,1};

  for(int a=0;a<8;a++) {
    bool pixVal1,pixVal2;
    //int newX1=pt(0)+xChange[a], newY1=pt(1)+yChange[a];
    //int newX2=pt(0)+xChange[a+1],newY2=pt(1)+yChange[a+1];

    if(image.contains(pt+ICoord(xChange[a],yChange[a])))
      pixVal1=image[pt+ICoord(xChange[a],yChange[a])];
    else
      pixVal1=0;
    
    if(image.contains(pt+ICoord(xChange[a+1],yChange[a+1])))
      pixVal2=image[pt+ICoord(xChange[a+1],yChange[a+1])];
    else
      pixVal2=0;
    if(pixVal1==0 && pixVal2==1)
      numTrans++;
  }
 
  if (numTrans!=1)
    return false;

  //check nonzero neighbors
  int numNonzero=0;
  for (int a=-1;a<=1;a++) {
    for (int b=-1;b<=1;b++) {
      if(image.contains(pt+ICoord(a,b)) && (a!=0 || b!=0) && image[pt+ICoord(a,b)])
	numNonzero++;
    }
  }
  if ((numNonzero<2) || (numNonzero>6))
    return false;

  return true;
}

bool Skeletonize::satisfiesStep1(const BoolArray &image,ICoord pt) {
//  if(pt(0)==0 || pt(0)==image.width()-1 || pt(1)==0 || pt(1)==image.height()-1)
//    return false;
  int p2=0,p4=0,p6=0,p8=0;
  if(pt(1)+1<image.height())
    p2=int(image[ICoord(pt(0),pt(1)+1)]);
  if(pt(0)+1<image.width())
    p4=int(image[ICoord(pt(0)+1,pt(1))]);
  if(pt(1)-1>=0)
    p6=int(image[ICoord(pt(0),pt(1)-1)]);
  if(pt(0)-1>=0)
    p8=int(image[ICoord(pt(0)-1,pt(1))]);
  if ((p2*p4*p6==0)&&(p4*p6*p8==0)) {
    return true;
  }    
  else {
    return false;
  }
}

bool Skeletonize::satisfiesStep2(const BoolArray &image,ICoord pt) {
//  if(pt(0)==0 || pt(0)==image.width()-1 || pt(1)==0 || pt(1)==image.height()-1)
//    return false;

  int p2=0,p4=0,p6=0,p8=0;
  if(pt(1)+1<image.height())
    p2=image[ICoord(pt(0),pt(1)+1)];
  if(pt(0)+1<image.width())
    p4=image[ICoord(pt(0)+1,pt(1))];
  if(pt(1)-1>=0)
    p6=image[ICoord(pt(0),pt(1)-1)];
  if(pt(0)-1>=0)
    p8=image[ICoord(pt(0)-1,pt(1))];
  if ((p2*p4*p8==0)&&(p2*p6*p8==0))
    return true;
  else
    return false;
}



DoubleArray Skeletonize::skeletonize(DoubleArray &im) {
  BoolArray image = BoolArray(im.size());
  for (DoubleArray::iterator i = im.begin();i !=im.end(); ++i){
    ICoord c = i.coord();
		if (im[c] == 0)
			image[c] = 0;
		else
			image[c] = 1;
	}
		
  int w=image.width();
  int h=image.height();
  BoolArray flag(ICoord(w,h),false);
  for(BoolArray::iterator i=flag.begin();i!=flag.end(); ++i)
    flag[i]=0;
  BoolArray deleted = image.clone();

  int step=1;
  bool stop=false;

  while (!stop) {
    int numChanged=0;
    for (int a = 0; a < w; a++) {
      for (int b = 0; b < h; b++) {
	if (isContour(deleted,ICoord(a,b)) && isDeletable(deleted,ICoord(a,b))) {
	  if (step == 1) {
	    if (satisfiesStep1(deleted,ICoord(a,b))) {
              flag[ICoord(a,b)]=1;
 	      numChanged++;
	    }
	  }
	  else {
	    if (satisfiesStep2(deleted,ICoord(a,b))) {  
	      flag[ICoord(a,b)]=1;  // Marks point for deletion
  	      numChanged++;
	    }
	  }
	}
      }
    }

    //remove flagged points
    for (int c = 0; c < w; c++) {
      for (int d = 0; d < h; d++) {
	if (flag[ICoord(c,d)]==1) {
	  flag[ICoord(c,d)]=0;
	  deleted[ICoord(c,d)]=0;
	}
      }
    }
    step *= -1;
    if (numChanged==0)
      stop=true;
  }
  DoubleArray toReturn = DoubleArray(deleted.size());
  for (BoolArray::iterator i = deleted.begin(); i != deleted.end(); ++i){
	ICoord c = i.coord();
	if (deleted[c] == 0)
		toReturn[c] = 0;
	else
		toReturn[c] = 1;
  }
  return toReturn;
}
